////////////////////////////////////////////////////////////////////////////////
///
///		Title		: Root Container
///
///		Copyright (c) 2006, NHN Co., Ltd. ALL RIGHTS RESERVED
///
///		Team		: Desktop Service Development
///
////////////////////////////////////////////////////////////////////////////////
#pragma once

#include "Container.h"
#include "NToolTipMgr.h"

/**
* @ingroup 	Base UI	
* @brief 	
* @date		10/13/2006
* @author	telltale
* @remark	
*/
class NPCOMMONUI_API CRootContainer : public CContainer
{
public:
	CRootContainer();
	virtual ~CRootContainer(void);
	/** {kr}
	* @fn void Init();
	* @brief ʱȭ @n
	*	 CommonUI Ʈѵ ʱȭ Ŀ ȣ.
	* {/kr} */
	void Init();
	/** {kr}
	* @fn void Close();
	* @brief {kr} ó @n
	*	Ŀ  CommonUI Ʈѵ ó.
	* {/kr} */
	void Close();
	/**
	* @brief 	the SetFocus function sets the keyboard focus to the specified component.
	* @param	pFocusCompoent	component to the window that will receive the keyboard input
	* @return	If the function succeeds, the return value is the handle to the component had the keyboard focus
	*/
	CComponent* SetFocus(CComponent* pFocusComponent);
	/**
	* @brief 	the GetFocus function retrieves the component that has the keyboard focus.
	* @return	the return value is the component with the keyboard focus.
	*/
	CComponent* GetFocus();
	/**
	* @brief 	the function sets the specified component below cursor
	* @param	pLastCursorComponent	component below cursor
	* @return	If the function succeeds, the return value is the handle to the last component
	*/
	CComponent* SetLastCursorComponent(CComponent* pLastCursorComponent);
	/**
	* @brief 	the function retrieves the component that has the last component below cursor
	* @return	the return value is the last component below cursor
	*/
	CComponent* GetLastCursorComponent();
	/**
	* @brief 	the function sets the specified component that has captured
	* @param	pCapturedComponent	component to capture
	* @return	If the function succeeds, the return values is TRUE
	*/
	BOOL	SetCapturedComponent(CComponent* pCapturedComponent);
	/**
	* @brief 	the function retrieves the captured component
	* @return	the return value is the current captured component
	*/
	CComponent*	GetCapturedComponent();
	/**
	* @brief 	the SetOwner function sets the owner window
	* @param	hWndOwner	attached window handle
	* @return	If the function succeeds, the return value is the handle of previous window
	*/
	HWND	SetOwner(HWND hWndOwner);
	/**
	* @brief 	the GetOwner function retrieves the handle of owner window
	* @return	the return value is the owner window
	*/
	HWND	GetOwner();
	/**
	* @brief {kr}	̽  1 Ŵ .
					 Ʈѵ ڽ Ʈ ̿ϴµ,
					Ʈѿ Ŵ ؼ ؾ ϱ ,
					Ŵ ܺο   ְ ͸ . {/kr}
	*/
	CNToolTipMgr* GetToolTipMgr();
	LPCTSTR GetClassName() { return _T("RootContainer"); }

	DECLARE_RTTI;

VBEGIN_MSG_MAP(CRootContainer)
	{	
		if (m_pCapturedComponent && 
			_IsMouseMessage(uMsg) == TRUE)
		{
			CHAIN_MSG_MAP_PTRMEMBER(m_pCapturedComponent)
			CHAIN_MSG_MAP(CComponent)
		}
		else if (m_pFocusedComponent)
		{
			CHAIN_MSG_MAP_PTRMEMBER(m_pFocusedComponent)
			CHAIN_MSG_MAP(CContainer)
		}
		else
		{
			CHAIN_MSG_MAP(CContainer)
		}
	}
END_MSG_MAP()

private:
	/// {kr}  ޽ 콺 ޽ . {/kr}@n
	BOOL _IsMouseMessage(UINT uMsg);

	///< focused component
	CComponent*	m_pFocusedComponent;
	///< current component below cursor
	CComponent* m_pLastCursorComponent;
	///< current capture component
	CComponent* m_pCapturedComponent;
	///< owner window handle
	HWND	m_hWndOwner;
	CNToolTipMgr m_ToolTipMgr;
};